+{
   locale_version => 1.12,
   variable => 'shifted',
   alternate => 'shifted',
   entry => <<'ENTRY', # for DUCET v7.0.0
0E2F      ; [*03D3.0020.0002][*FFF0.0000.0000] # THAI CHARACTER PAIYANNOI
0E5A      ; [*03D3.0020.0002][*FFF1.0000.0000] # THAI CHARACTER ANGKHANKHU
0E46      ; [*03D4.0020.0002][*FFF1.0000.0000] # THAI CHARACTER MAIYAMOK
0E5B      ; [*03D4.0020.0002][*FFF0.0000.0000] # THAI CHARACTER KHOMUT
0E4C      ; [.0000.00CA.0002] # THAI CHARACTER THANTHAKHAT
0E47      ; [.0000.00CB.0002] # THAI CHARACTER MAITAIKHU
0E48      ; [.0000.00CC.0002] # THAI CHARACTER MAI EK
0E49      ; [.0000.00CD.0002] # THAI CHARACTER MAI THO
0E4A      ; [.0000.00CE.0002] # THAI CHARACTER MAI TRI
0E4B      ; [.0000.00CF.0002] # THAI CHARACTER MAI CHATTAWA
0E4D      ; [.29CB.0020.0002] # THAI CHARACTER NIKHAHIT
0E45      ; [.29CE.0020.0003] # THAI CHARACTER LAKKHANGYAO
0E4D 0E32 ; [.29CE.0020.0002][.29CB.0020.0003] # THAI CHARACTER SARA AM
0E33      ; [.29CE.0020.0002][.29CB.0020.0004] # THAI CHARACTER SARA AM
0E45 0E4D ; [.29CB.0020.0002][.29CE.0020.0004] # <THAI CHARACTER LAKKHANGYAO, THAI CHARACTER NIKHAHIT>
0E3A      ; [.29DC.0020.0002] # THAI CHARACTER PHINTHU
ENTRY
};
