+{
   locale_version => 1.12,
   entry => <<'ENTRY', # for DUCET v7.0.0
0161      ; [.1B54.0020.0002] # LATIN SMALL LETTER S WITH CARON
0073 030C ; [.1B54.0020.0002] # LATIN SMALL LETTER S WITH CARON
0160      ; [.1B54.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
0053 030C ; [.1B54.0020.0008] # LATIN CAPITAL LETTER S WITH CARON
007A      ; [.1B55.0020.0002] # LATIN SMALL LETTER Z
005A      ; [.1B55.0020.0008] # LATIN CAPITAL LETTER Z
017E      ; [.1B56.0020.0002] # LATIN SMALL LETTER Z WITH CARON
007A 030C ; [.1B56.0020.0002] # LATIN SMALL LETTER Z WITH CARON
017D      ; [.1B56.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
005A 030C ; [.1B56.0020.0008] # LATIN CAPITAL LETTER Z WITH CARON
0077      ; [.1BA5.0021.0002] # LATIN SMALL LETTER W
0057      ; [.1BA5.0021.0008] # LATIN CAPITAL LETTER W
00F5      ; [.1BBD.0020.0002] # LATIN SMALL LETTER O WITH TILDE
006F 0303 ; [.1BBD.0020.0002] # LATIN SMALL LETTER O WITH TILDE
00D5      ; [.1BBD.0020.0008] # LATIN CAPITAL LETTER O WITH TILDE
004F 0303 ; [.1BBD.0020.0008] # LATIN CAPITAL LETTER O WITH TILDE
1E4D      ; [.1BBD.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER O WITH TILDE AND ACUTE
1E4C      ; [.1BBD.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER O WITH TILDE AND ACUTE
022D      ; [.1BBD.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER O WITH TILDE AND MACRON
022C      ; [.1BBD.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER O WITH TILDE AND MACRON
1E4F      ; [.1BBD.0020.0002][.0000.002B.0002] # LATIN SMALL LETTER O WITH TILDE AND DIAERESIS
1E4E      ; [.1BBD.0020.0008][.0000.002B.0002] # LATIN CAPITAL LETTER O WITH TILDE AND DIAERESIS
1EE1      ; [.1BBD.0020.0002][.0000.003F.0002] # LATIN SMALL LETTER O WITH HORN AND TILDE
1EE0      ; [.1BBD.0020.0008][.0000.003F.0002] # LATIN CAPITAL LETTER O WITH HORN AND TILDE
00E4      ; [.1BBE.0020.0002] # LATIN SMALL LETTER A WITH DIAERESIS
0061 0308 ; [.1BBE.0020.0002] # LATIN SMALL LETTER A WITH DIAERESIS
00C4      ; [.1BBE.0020.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
0041 0308 ; [.1BBE.0020.0008] # LATIN CAPITAL LETTER A WITH DIAERESIS
01DF      ; [.1BBE.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER A WITH DIAERESIS AND MACRON
01DE      ; [.1BBE.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER A WITH DIAERESIS AND MACRON
00F6      ; [.1BBF.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
006F 0308 ; [.1BBF.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
00D6      ; [.1BBF.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
004F 0308 ; [.1BBF.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
022B      ; [.1BBF.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
022A      ; [.1BBF.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
00FC      ; [.1BC0.0020.0002] # LATIN SMALL LETTER U WITH DIAERESIS
0075 0308 ; [.1BC0.0020.0002] # LATIN SMALL LETTER U WITH DIAERESIS
00DC      ; [.1BC0.0020.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
0055 0308 ; [.1BC0.0020.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
01DC      ; [.1BC0.0020.0002][.0000.0025.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
01DB      ; [.1BC0.0020.0008][.0000.0025.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE
01D8      ; [.1BC0.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
01D7      ; [.1BC0.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE
01D6      ; [.1BC0.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
01D5      ; [.1BC0.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON
01DA      ; [.1BC0.0020.0002][.0000.0028.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND CARON
01D9      ; [.1BC0.0020.0008][.0000.0028.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON
ENTRY
};
