+{
   locale_version => 1.12,
   entry => <<'ENTRY', # for DUCET v7.0.0
00F1      ; [.1A7E.0020.0002] # LATIN SMALL LETTER N WITH TILDE
006E 0303 ; [.1A7E.0020.0002] # LATIN SMALL LETTER N WITH TILDE
00D1      ; [.1A7E.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
004E 0303 ; [.1A7E.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
006E 0067 ; [.1A7F.0020.0002] # <LATIN SMALL LETTER N, LATIN SMALL LETTER G>
004E 0067 ; [.1A7F.0020.0007] # <LATIN CAPITAL LETTER N, LATIN SMALL LETTER G>
004E 0047 ; [.1A7F.0020.0008] # <LATIN CAPITAL LETTER N, LATIN CAPITAL LETTER G>
ENTRY
};
