+{
   locale_version => 1.12,
   entry => <<'ENTRY', # for DUCET v7.0.0
00E7      ; [.193F.0020.0002] # LATIN SMALL LETTER C WITH CEDILLA
0063 0327 ; [.193F.0020.0002] # LATIN SMALL LETTER C WITH CEDILLA
00C7      ; [.193F.0020.0008] # LATIN CAPITAL LETTER C WITH CEDILLA
0043 0327 ; [.193F.0020.0008] # LATIN CAPITAL LETTER C WITH CEDILLA
011F      ; [.19B9.0020.0002] # LATIN SMALL LETTER G WITH BREVE
0067 0306 ; [.19B9.0020.0002] # LATIN SMALL LETTER G WITH BREVE
011E      ; [.19B9.0020.0008] # LATIN CAPITAL LETTER G WITH BREVE
0047 0306 ; [.19B9.0020.0008] # LATIN CAPITAL LETTER G WITH BREVE
0131      ; [.19F5.0020.0002] # LATIN SMALL LETTER DOTLESS I
0049      ; [.19F5.0020.0008] # LATIN CAPITAL LETTER I
00CC      ; [.19F5.0020.0008][.0000.0025.0002] # LATIN CAPITAL LETTER I WITH GRAVE
00CD      ; [.19F5.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER I WITH ACUTE
00CE      ; [.19F5.0020.0008][.0000.0027.0002] # LATIN CAPITAL LETTER I WITH CIRCUMFLEX
00CF      ; [.19F5.0020.0008][.0000.002B.0002] # LATIN CAPITAL LETTER I WITH DIAERESIS
012A      ; [.19F5.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER I WITH MACRON
012C      ; [.19F5.0020.0008][.0000.0026.0002] # LATIN CAPITAL LETTER I WITH BREVE
012E      ; [.19F5.0020.0008][.0000.0031.0002] # LATIN CAPITAL LETTER I WITH OGONEK
0130      ; [.19F6.0020.0008] # LATIN CAPITAL LETTER I WITH DOT ABOVE
0049 0307 ; [.19F6.0020.0008] # LATIN CAPITAL LETTER I WITH DOT ABOVE
00F6      ; [.1AA2.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
006F 0308 ; [.1AA2.0020.0002] # LATIN SMALL LETTER O WITH DIAERESIS
00D6      ; [.1AA2.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
004F 0308 ; [.1AA2.0020.0008] # LATIN CAPITAL LETTER O WITH DIAERESIS
022B      ; [.1AA2.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER O WITH DIAERESIS AND MACRON
022A      ; [.1AA2.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER O WITH DIAERESIS AND MACRON
015F      ; [.1B34.0020.0002] # LATIN SMALL LETTER S WITH CEDILLA
0073 0327 ; [.1B34.0020.0002] # LATIN SMALL LETTER S WITH CEDILLA
015E      ; [.1B34.0020.0008] # LATIN CAPITAL LETTER S WITH CEDILLA
0053 0327 ; [.1B34.0020.0008] # LATIN CAPITAL LETTER S WITH CEDILLA
00FC      ; [.1B78.0020.0002] # LATIN SMALL LETTER U WITH DIAERESIS
0075 0308 ; [.1B78.0020.0002] # LATIN SMALL LETTER U WITH DIAERESIS
00DC      ; [.1B78.0020.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
0055 0308 ; [.1B78.0020.0008] # LATIN CAPITAL LETTER U WITH DIAERESIS
01DC      ; [.1B78.0020.0002][.0000.0025.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND GRAVE
01DB      ; [.1B78.0020.0008][.0000.0025.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND GRAVE
01D8      ; [.1B78.0020.0002][.0000.0024.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND ACUTE
01D7      ; [.1B78.0020.0008][.0000.0024.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND ACUTE
01D6      ; [.1B78.0020.0002][.0000.0032.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND MACRON
01D5      ; [.1B78.0020.0008][.0000.0032.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND MACRON
01DA      ; [.1B78.0020.0002][.0000.0028.0002] # LATIN SMALL LETTER U WITH DIAERESIS AND CARON
01D9      ; [.1B78.0020.0008][.0000.0028.0002] # LATIN CAPITAL LETTER U WITH DIAERESIS AND CARON
ENTRY
};
