+{
   locale_version => 1.12,
   entry => <<'ENTRY', # for DUCET v7.0.0
0629      ; [.2025.0021.0002] # ARABIC LETTER TEH MARBUTA
FE94      ; [.2025.0021.0019] # ARABIC LETTER TEH MARBUTA FINAL FORM
FE93      ; [.2025.0021.001A] # ARABIC LETTER TEH MARBUTA ISOLATED FORM
0649      ; [.20C7.0021.0002] # ARABIC LETTER ALEF MAKSURA
FBE8      ; [.20C7.0021.0017] # ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA INITIAL FORM
FBE9      ; [.20C7.0021.0018] # ARABIC LETTER UIGHUR KAZAKH KIRGHIZ ALEF MAKSURA MEDIAL FORM
FEF0      ; [.20C7.0021.0019] # ARABIC LETTER ALEF MAKSURA FINAL FORM
FEEF      ; [.20C7.0021.001A] # ARABIC LETTER ALEF MAKSURA ISOLATED FORM
FC90      ; [.20C7.0021.001B] # ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF FINAL FORM
FC5D      ; [.20C7.0021.001C] # ARABIC LIGATURE ALEF MAKSURA WITH SUPERSCRIPT ALEF ISOLATED FORM
ENTRY
};
