+{
   locale_version => 1.12,
   entry => <<'ENTRY', # for DUCET v7.0.0
0063 0068 ; [.193F.0020.0002] # <LATIN SMALL LETTER C, LATIN SMALL LETTER H>
0043 0068 ; [.193F.0020.0007] # <LATIN CAPITAL LETTER C, LATIN SMALL LETTER H>
0043 0048 ; [.193F.0020.0008] # <LATIN CAPITAL LETTER C, LATIN CAPITAL LETTER H>
006C 006C ; [.1A3C.0020.0002] # <LATIN SMALL LETTER L, LATIN SMALL LETTER L>
004C 006C ; [.1A3C.0020.0007] # <LATIN CAPITAL LETTER L, LATIN SMALL LETTER L>
004C 004C ; [.1A3C.0020.0008] # <LATIN CAPITAL LETTER L, LATIN CAPITAL LETTER L>
00F1      ; [.1A7E.0020.0002] # LATIN SMALL LETTER N WITH TILDE
006E 0303 ; [.1A7E.0020.0002] # LATIN SMALL LETTER N WITH TILDE
00D1      ; [.1A7E.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
004E 0303 ; [.1A7E.0020.0008] # LATIN CAPITAL LETTER N WITH TILDE
ENTRY
};
