package #
Locale::Codes::Country_Retired;

# This file was automatically generated.  Any changes to this file will
# be lost the next time 'deprecate_codes' is run.
#    Generated on: Fri Feb 20 15:27:22 GMT 2015

use strict;
require 5.006;
use warnings;
use utf8;

our($VERSION);
$VERSION='3.34';

$Locale::Codes::Retired{'country'}{'alpha-2'}{'code'} = {
   q(an) => q(Netherlands Antilles),
   q(cs) => q(Serbia and Montenegro),
   q(fx) => q(France, Metropolitan),
   q(tp) => q(East Timor),
   q(yu) => q(Yugoslavia),
   q(zr) => q(Zaire),
};

$Locale::Codes::Retired{'country'}{'alpha-3'}{'code'} = {
   q(ant) => q(Netherlands Antilles),
   q(ata) => q(Antarctica),
   q(atf) => q(French Southern and Antarctic Lands),
   q(bvt) => q(Bouvet Island),
   q(cck) => q(Cocos (Keeling) Islands),
   q(cxr) => q(Christmas Island),
   q(fxx) => q(France, Metropolitan),
   q(hmd) => q(Heard Island and Mcdonald Islands),
   q(iot) => q(British Indian Ocean Territory),
   q(rom) => q(Romania),
   q(scg) => q(Serbia and Montenegro),
   q(sgs) => q(South Georgia and the Islands),
   q(tmp) => q(East Timor),
   q(twn) => q(Taiwan),
   q(umi) => q(United States Minor Outlying Islands),
   q(yug) => q(Yugoslavia),
   q(zar) => q(Zaire),
};

$Locale::Codes::Retired{'country'}{'dom'}{'code'} = {
   q(AC) => q(Ascension Island),
   q(AD) => q(Andorra),
   q(AE) => q(United Arab Emirates),
   q(AF) => q(Afghanistan),
   q(AG) => q(Antigua and Barbuda),
   q(AI) => q(Anguilla),
   q(AL) => q(Albania),
   q(AM) => q(Armenia),
   q(AN) => q(Netherlands Antilles),
   q(AO) => q(Angola),
   q(AQ) => q(Antarctica),
   q(AR) => q(Argentina),
   q(AS) => q(American Samoa),
   q(AT) => q(Austria),
   q(AU) => q(Australia),
   q(AW) => q(Aruba),
   q(AX) => q(Aland Islands),
   q(AZ) => q(Azerbaijan),
   q(BA) => q(Bosnia and Herzegovina),
   q(BB) => q(Barbados),
   q(BD) => q(Bangladesh),
   q(BE) => q(Belgium),
   q(BF) => q(Burkina Faso),
   q(BG) => q(Bulgaria),
   q(BH) => q(Bahrain),
   q(BI) => q(Burundi),
   q(BJ) => q(Benin),
   q(BL) => q(Saint Barthelemy),
   q(BM) => q(Bermuda),
   q(BN) => q(Brunei Darussalam),
   q(BO) => q(Bolivia, Plurinational State of),
   q(BQ) => q(Bonaire, Sint Eustatius and Saba),
   q(BR) => q(Brazil),
   q(BS) => q(Bahamas),
   q(BT) => q(Bhutan),
   q(BV) => q(Bouvet Island),
   q(BW) => q(Botswana),
   q(BY) => q(Belarus),
   q(BZ) => q(Belize),
   q(CA) => q(Canada),
   q(CC) => q(Cocos (Keeling) Islands),
   q(CD) => q(Congo, The Democratic Republic of the),
   q(CF) => q(Central African Republic),
   q(CG) => q(Congo),
   q(CH) => q(Switzerland),
   q(CI) => q(Cote d'Ivoire),
   q(CK) => q(Cook Islands),
   q(CL) => q(Chile),
   q(CM) => q(Cameroon),
   q(CN) => q(China),
   q(CO) => q(Colombia),
   q(CR) => q(Costa Rica),
   q(CU) => q(Cuba),
   q(CV) => q(Cape Verde),
   q(CW) => q(Curacao),
   q(CX) => q(Christmas Island),
   q(CY) => q(Cyprus),
   q(CZ) => q(Czech Republic),
   q(DE) => q(Germany),
   q(DJ) => q(Djibouti),
   q(DK) => q(Denmark),
   q(DM) => q(Dominica),
   q(DO) => q(Dominican Republic),
   q(DZ) => q(Algeria),
   q(EC) => q(Ecuador),
   q(EE) => q(Estonia),
   q(EG) => q(Egypt),
   q(EH) => q(Western Sahara),
   q(ER) => q(Eritrea),
   q(ES) => q(Spain),
   q(ET) => q(Ethiopia),
   q(EU) => q(European Union),
   q(FI) => q(Finland),
   q(FJ) => q(Fiji),
   q(FK) => q(Falkland Islands (Malvinas)),
   q(FM) => q(Micronesia, Federated States of),
   q(FO) => q(Faroe Islands),
   q(FR) => q(France),
   q(FX) => q(France, Metropolitan),
   q(GA) => q(Gabon),
   q(GB) => q(United Kingdom),
   q(GD) => q(Grenada),
   q(GE) => q(Georgia),
   q(GF) => q(French Guiana),
   q(GG) => q(Guernsey),
   q(GH) => q(Ghana),
   q(GI) => q(Gibraltar),
   q(GL) => q(Greenland),
   q(GM) => q(Gambia),
   q(GN) => q(Guinea),
   q(GP) => q(Guadeloupe),
   q(GQ) => q(Equatorial Guinea),
   q(GR) => q(Greece),
   q(GS) => q(South Georgia and the South Sandwich Islands),
   q(GT) => q(Guatemala),
   q(GU) => q(Guam),
   q(GW) => q(Guinea-Bissau),
   q(GY) => q(Guyana),
   q(HK) => q(Hong Kong),
   q(HM) => q(Heard Island and Mcdonald Islands),
   q(HN) => q(Honduras),
   q(HR) => q(Croatia),
   q(HT) => q(Haiti),
   q(HU) => q(Hungary),
   q(ID) => q(Indonesia),
   q(IE) => q(Ireland),
   q(IL) => q(Israel),
   q(IM) => q(Isle of Man),
   q(IN) => q(India),
   q(IO) => q(British Indian Ocean Territory),
   q(IQ) => q(Iraq),
   q(IR) => q(Iran, Islamic Republic of),
   q(IS) => q(Iceland),
   q(IT) => q(Italy),
   q(JE) => q(Jersey),
   q(JM) => q(Jamaica),
   q(JO) => q(Jordan),
   q(JP) => q(Japan),
   q(KE) => q(Kenya),
   q(KG) => q(Kyrgyzstan),
   q(KH) => q(Cambodia),
   q(KI) => q(Kiribati),
   q(KM) => q(Comoros),
   q(KN) => q(Saint Kitts and Nevis),
   q(KP) => q(Korea, Democratic People's Republic of),
   q(KR) => q(Korea, Republic of),
   q(KW) => q(Kuwait),
   q(KY) => q(Cayman Islands),
   q(KZ) => q(Kazakhstan),
   q(LA) => q(Lao People's Democratic Republic),
   q(LB) => q(Lebanon),
   q(LC) => q(Saint Lucia),
   q(LI) => q(Liechtenstein),
   q(LK) => q(Sri Lanka),
   q(LR) => q(Liberia),
   q(LS) => q(Lesotho),
   q(LT) => q(Lithuania),
   q(LU) => q(Luxembourg),
   q(LV) => q(Latvia),
   q(LY) => q(Libya),
   q(MA) => q(Morocco),
   q(MC) => q(Monaco),
   q(MD) => q(Moldova, Republic of),
   q(ME) => q(Montenegro),
   q(MF) => q(Saint Martin (French part)),
   q(MG) => q(Madagascar),
   q(MH) => q(Marshall Islands),
   q(MK) => q(Macedonia, The Former Yugoslav Republic of),
   q(ML) => q(Mali),
   q(MM) => q(Myanmar),
   q(MN) => q(Mongolia),
   q(MO) => q(Macao),
   q(MP) => q(Northern Mariana Islands),
   q(MQ) => q(Martinique),
   q(MR) => q(Mauritania),
   q(MS) => q(Montserrat),
   q(MT) => q(Malta),
   q(MU) => q(Mauritius),
   q(MV) => q(Maldives),
   q(MW) => q(Malawi),
   q(MX) => q(Mexico),
   q(MY) => q(Malaysia),
   q(MZ) => q(Mozambique),
   q(NA) => q(Namibia),
   q(NC) => q(New Caledonia),
   q(NE) => q(Niger),
   q(NF) => q(Norfolk Island),
   q(NG) => q(Nigeria),
   q(NI) => q(Nicaragua),
   q(NL) => q(Netherlands),
   q(NO) => q(Norway),
   q(NP) => q(Nepal),
   q(NR) => q(Nauru),
   q(NU) => q(Niue),
   q(NZ) => q(New Zealand),
   q(OM) => q(Oman),
   q(PA) => q(Panama),
   q(PE) => q(Peru),
   q(PF) => q(French Polynesia),
   q(PG) => q(Papua New Guinea),
   q(PH) => q(Philippines),
   q(PK) => q(Pakistan),
   q(PL) => q(Poland),
   q(PM) => q(Saint Pierre and Miquelon),
   q(PN) => q(Pitcairn),
   q(PR) => q(Puerto Rico),
   q(PS) => q(Palestinian Territory, Occupied),
   q(PT) => q(Portugal),
   q(PW) => q(Palau),
   q(PY) => q(Paraguay),
   q(QA) => q(Qatar),
   q(RE) => q(Reunion),
   q(RO) => q(Romania),
   q(RS) => q(Serbia),
   q(RU) => q(Russian Federation),
   q(RW) => q(Rwanda),
   q(SA) => q(Saudi Arabia),
   q(SB) => q(Solomon Islands),
   q(SC) => q(Seychelles),
   q(SD) => q(Sudan),
   q(SE) => q(Sweden),
   q(SG) => q(Singapore),
   q(SH) => q(Saint Helena, Ascension and Tristan da Cunha),
   q(SI) => q(Slovenia),
   q(SJ) => q(Svalbard and Jan Mayen),
   q(SK) => q(Slovakia),
   q(SL) => q(Sierra Leone),
   q(SM) => q(San Marino),
   q(SN) => q(Senegal),
   q(SO) => q(Somalia),
   q(SR) => q(Suriname),
   q(SS) => q(South Sudan),
   q(ST) => q(Sao Tome and Principe),
   q(SU) => q(Soviet Union),
   q(SV) => q(El Salvador),
   q(SX) => q(Sint Maarten (Dutch part)),
   q(SY) => q(Syrian Arab Republic),
   q(SZ) => q(Swaziland),
   q(TC) => q(Turks and Caicos Islands),
   q(TD) => q(Chad),
   q(TF) => q(French Southern Territories),
   q(TG) => q(Togo),
   q(TH) => q(Thailand),
   q(TJ) => q(Tajikistan),
   q(TK) => q(Tokelau),
   q(TL) => q(Timor-Leste),
   q(TM) => q(Turkmenistan),
   q(TN) => q(Tunisia),
   q(TO) => q(Tonga),
   q(TP) => q(Portuguese Timor),
   q(TR) => q(Turkey),
   q(TT) => q(Trinidad and Tobago),
   q(TV) => q(Tuvalu),
   q(TW) => q(Taiwan, Province of China),
   q(TZ) => q(Tanzania, United Republic of),
   q(UA) => q(Ukraine),
   q(UG) => q(Uganda),
   q(UK) => q(United Kingdom),
   q(UM) => q(United States Minor Outlying Islands),
   q(US) => q(United States),
   q(UY) => q(Uruguay),
   q(UZ) => q(Uzbekistan),
   q(VA) => q(Holy See (Vatican City State)),
   q(VC) => q(Saint Vincent and the Grenadines),
   q(VE) => q(Venezuela, Bolivarian Republic of),
   q(VG) => q(Virgin Islands, British),
   q(VI) => q(Virgin Islands, U.S.),
   q(VN) => q(Viet Nam),
   q(VU) => q(Vanuatu),
   q(WF) => q(Wallis and Futuna),
   q(WS) => q(Samoa),
   q(YE) => q(Yemen),
   q(YT) => q(Mayotte),
   q(YU) => q(Yugoslavia ),
   q(ZA) => q(South Africa),
   q(ZM) => q(Zambia),
   q(ZW) => q(Zimbabwe),
   q(ac) => q(Ascension Island),
   q(an) => q(Netherlands Antilles),
   q(eu) => q(European Union),
   q(su) => q(USSR),
   q(tp) => q(East Timor),
   q(uk) => q(United Kingdom),
};

$Locale::Codes::Retired{'country'}{'numeric'}{'code'} = {
   q(010) => q(Antarctica),
   q(074) => q(Bouvet Island),
   q(086) => q(British Indian Ocean Territory),
   q(158) => q(Taiwan),
   q(162) => q(Christmas Island),
   q(166) => q(Cocos (Keeling) Islands),
   q(239) => q(South Georgia and the Islands),
   q(249) => q(France, Metropolitan),
   q(260) => q(French Southern and Antarctic Lands),
   q(334) => q(Heard Island and Mcdonald Islands),
   q(530) => q(Netherlands Antilles),
   q(581) => q(United States Minor Outlying Islands),
   q(680) => q(Sark),
   q(736) => q(Sudan),
   q(830) => q(Channel Islands),
   q(891) => q(Serbia and Montenegro),
};

$Locale::Codes::Retired{'country'}{'alpha-2'}{'name'} = {
   q(bolivia) => [ q(bo), q(Bolivia) ],
   q(bolivia, plurinational state of) => [ q(bo), q(Bolivia, Plurinational State of) ],
   q(bonaire, saint eustatius and saba) => [ q(bq), q(Bonaire, Saint Eustatius and Saba) ],
   q(bosnia and herzegowina) => [ q(ba), q(Bosnia and Herzegowina) ],
   q(cape verde) => [ q(cv), q(Cape Verde) ],
   q(congo, the democratic republic of the) => [ q(cd), q(Congo, The Democratic Republic of the) ],
   q(east timor) => [ q(tl), q(East Timor) ],
   q(falkland islands (malvinas)) => [ q(fk), q(Falkland Islands (Malvinas)) ],
   q(france, metropolitan) => [ q(fx), q(France, Metropolitan) ],
   q(heard and mc donald islands) => [ q(hm), q(Heard and Mc Donald Islands) ],
   q(holy see (the) [vatican city state]) => [ q(va), q(Holy See (The) [Vatican City State]) ],
   q(holy see (vatican city state)) => [ q(va), q(Holy See (Vatican City State)) ],
   q(iran (islamic republic of)) => [ q(ir), q(Iran (Islamic Republic of)) ],
   q(iran, islamic republic of) => [ q(ir), q(Iran, Islamic Republic of) ],
   q(iran, the islamic republic of) => [ q(ir), q(Iran, The Islamic Republic of) ],
   q(kazakstan) => [ q(kz), q(Kazakstan) ],
   q(korea, democratic people's republic of) => [ q(kp), q(Korea, Democratic People's Republic of) ],
   q(korea, republic of) => [ q(kr), q(Korea, Republic of) ],
   q(libyan arab jamahiriya) => [ q(ly), q(Libyan Arab Jamahiriya) ],
   q(macau) => [ q(mo), q(Macau) ],
   q(micronesia (federated states of)) => [ q(fm), q(Micronesia (Federated States of)) ],
   q(micronesia, federated states of) => [ q(fm), q(Micronesia, Federated States of) ],
   q(micronesia, the federated states of) => [ q(fm), q(Micronesia, The Federated States of) ],
   q(moldova, republic of) => [ q(md), q(Moldova, Republic of) ],
   q(netherlands antilles) => [ q(an), q(Netherlands Antilles) ],
   q(saint helena) => [ q(sh), q(Saint Helena) ],
   q(saint martin) => [ q(mf), q(Saint Martin) ],
   q(serbia and montenegro) => [ q(cs), q(Serbia and Montenegro) ],
   q(st. helena) => [ q(sh), q(St. Helena) ],
   q(st. pierre and miquelon) => [ q(pm), q(St. Pierre and Miquelon) ],
   q(svalbard and jan mayen islands) => [ q(sj), q(Svalbard and Jan Mayen Islands) ],
   q(taiwan, province of china) => [ q(tw), q(Taiwan, Province of China) ],
   q(united kingdom) => [ q(gb), q(United Kingdom) ],
   q(united states) => [ q(us), q(United States) ],
   q(vatican city state (holy see)) => [ q(va), q(Vatican City State (Holy See)) ],
   q(venezuela) => [ q(ve), q(Venezuela) ],
   q(venezuela, bolivarian republic of) => [ q(ve), q(Venezuela, Bolivarian Republic of) ],
   q(venezuela, bolivarian republic of ) => [ q(ve), q(Venezuela, Bolivarian Republic of ) ],
   q(vietnam) => [ q(vn), q(Vietnam) ],
   q(virgin islands (british)) => [ q(vg), q(Virgin Islands (British)) ],
   q(virgin islands (u.s.)) => [ q(vi), q(Virgin Islands (U.S.)) ],
   q(virgin islands, british) => [ q(vg), q(Virgin Islands, British) ],
   q(virgin islands, u.s.) => [ q(vi), q(Virgin Islands, U.S.) ],
   q(wallis and futuna islands) => [ q(wf), q(Wallis and Futuna Islands) ],
   q(yugoslavia) => [ q(yu), q(Yugoslavia) ],
   q(zaire) => [ q(zr), q(Zaire) ],
};

$Locale::Codes::Retired{'country'}{'alpha-3'}{'name'} = {
   q(antarctica) => [ q(ata), q(Antarctica) ],
   q(bolivia) => [ q(bol), q(Bolivia) ],
   q(bolivia (plurinational state of)) => [ q(bol), q(Bolivia (Plurinational State of)) ],
   q(bolivia, plurinational state of) => [ q(bol), q(Bolivia, Plurinational State of) ],
   q(bonaire, saint eustatius and saba) => [ q(bes), q(Bonaire, Saint Eustatius and Saba) ],
   q(bouvet island) => [ q(bvt), q(Bouvet Island) ],
   q(british indian ocean territory) => [ q(iot), q(British Indian Ocean Territory) ],
   q(british virgin islands) => [ q(vgb), q(British Virgin Islands) ],
   q(cabo verde) => [ q(cpv), q(Cabo Verde) ],
   q(cape verde) => [ q(cpv), q(Cape Verde) ],
   q(china, hong kong special administrative region) => [ q(hkg), q(China, Hong Kong Special Administrative Region) ],
   q(china, macao special administrative region) => [ q(mac), q(China, Macao Special Administrative Region) ],
   q(christmas island) => [ q(cxr), q(Christmas Island) ],
   q(cocos (keeling) islands) => [ q(cck), q(Cocos (Keeling) Islands) ],
   q(congo, the democratic republic of the) => [ q(cod), q(Congo, The Democratic Republic of the) ],
   q(democratic people's republic of korea) => [ q(prk), q(Democratic People's Republic of Korea) ],
   q(democratic republic of the congo) => [ q(cod), q(Democratic Republic of the Congo) ],
   q(east timor) => [ q(tls), q(East Timor) ],
   q(faeroe islands) => [ q(fro), q(Faeroe Islands) ],
   q(falkland islands (malvinas)) => [ q(flk), q(Falkland Islands (Malvinas)) ],
   q(faroe islands) => [ q(fro), q(Faroe Islands) ],
   q(france, metropolitan) => [ q(fxx), q(France, Metropolitan) ],
   q(french southern and antarctic lands) => [ q(atf), q(French Southern and Antarctic Lands) ],
   q(french southern territories) => [ q(atf), q(French Southern Territories) ],
   q(heard island and mcdonald islands) => [ q(hmd), q(Heard Island and Mcdonald Islands) ],
   q(holy see) => [ q(vat), q(Holy See) ],
   q(holy see (the) [vatican city state]) => [ q(vat), q(Holy See (The) [Vatican City State]) ],
   q(holy see (vatican city state)) => [ q(vat), q(Holy See (Vatican City State)) ],
   q(hong kong) => [ q(hkg), q(Hong Kong) ],
   q(hong kong special administrative region of china) => [ q(hkg), q(Hong Kong Special Administrative Region of China) ],
   q(iran (islamic republic of)) => [ q(irn), q(Iran (Islamic Republic of)) ],
   q(iran, islamic republic of) => [ q(irn), q(Iran, Islamic Republic of) ],
   q(iran, the islamic republic of) => [ q(irn), q(Iran, The Islamic Republic of) ],
   q(kazakstan) => [ q(kaz), q(Kazakstan) ],
   q(korea, democratic people's republic of) => [ q(prk), q(Korea, Democratic People's Republic of) ],
   q(korea, republic of) => [ q(kor), q(Korea, Republic of) ],
   q(libyan arab jamahiriya) => [ q(lby), q(Libyan Arab Jamahiriya) ],
   q(macao) => [ q(mac), q(Macao) ],
   q(macao special administrative region of china) => [ q(mac), q(Macao Special Administrative Region of China) ],
   q(macau) => [ q(mac), q(Macau) ],
   q(macedonia, the former yugoslav republic of) => [ q(mkd), q(Macedonia, the Former Yugoslav Republic of) ],
   q(micronesia (federated states of)) => [ q(fsm), q(Micronesia (Federated States of)) ],
   q(micronesia, federated states of) => [ q(fsm), q(Micronesia, Federated States of) ],
   q(micronesia, the federated states of) => [ q(fsm), q(Micronesia, The Federated States of) ],
   q(moldova, republic of) => [ q(mda), q(Moldova, Republic of) ],
   q(netherlands antilles) => [ q(ant), q(Netherlands Antilles) ],
   q(palestinian territory, occupied) => [ q(pse), q(Palestinian Territory, Occupied) ],
   q(republic of korea) => [ q(kor), q(Republic of Korea) ],
   q(republic of moldova) => [ q(mda), q(Republic of Moldova) ],
   q(saint helena) => [ q(shn), q(Saint Helena) ],
   q(saint-barthelemy) => [ q(blm), q(Saint-Barthelemy) ],
   q(saint-martin (french part)) => [ q(maf), q(Saint-Martin (French part)) ],
   q(serbia and montenegro) => [ q(scg), q(Serbia and Montenegro) ],
   q(south georgia and the islands) => [ q(sgs), q(South Georgia and the Islands) ],
   q(south georgia and the south sandwich islands) => [ q(sgs), q(South Georgia and the South Sandwich Islands) ],
   q(state of palestine) => [ q(pse), q(State of Palestine) ],
   q(svalbard and jan mayen) => [ q(sjm), q(Svalbard and Jan Mayen) ],
   q(svalbard and jan mayen islands) => [ q(sjm), q(Svalbard and Jan Mayen Islands) ],
   q(taiwan) => [ q(twn), q(Taiwan) ],
   q(taiwan, province of china) => [ q(twn), q(Taiwan, Province of China) ],
   q(tanzania, united republic of) => [ q(tza), q(Tanzania, United Republic of) ],
   q(the former yugoslav republic of macedonia) => [ q(mkd), q(The former Yugoslav Republic of Macedonia) ],
   q(united kingdom) => [ q(gbr), q(United Kingdom) ],
   q(united kingdom of great britain and northern ireland) => [ q(gbr), q(United Kingdom of Great Britain and Northern Ireland) ],
   q(united republic of tanzania) => [ q(tza), q(United Republic of Tanzania) ],
   q(united states) => [ q(usa), q(United States) ],
   q(united states minor outlying islands) => [ q(umi), q(United States Minor Outlying Islands) ],
   q(united states of america) => [ q(usa), q(United States of America) ],
   q(united states virgin islands) => [ q(vir), q(United States Virgin Islands) ],
   q(venezuela) => [ q(ven), q(Venezuela) ],
   q(venezuela (bolivarian republic of)) => [ q(ven), q(Venezuela (Bolivarian Republic of)) ],
   q(venezuela, bolivarian republic of ) => [ q(ven), q(Venezuela, Bolivarian Republic of ) ],
   q(vietnam) => [ q(vnm), q(Vietnam) ],
   q(virgin islands, british) => [ q(vgb), q(Virgin Islands, British) ],
   q(virgin islands, u.s.) => [ q(vir), q(Virgin Islands, U.S.) ],
   q(wallis and futuna) => [ q(wlf), q(Wallis and Futuna) ],
   q(wallis and futuna islands) => [ q(wlf), q(Wallis and Futuna Islands) ],
   q(yugoslavia) => [ q(yug), q(Yugoslavia) ],
   q(zaire) => [ q(zar), q(Zaire) ],
};

$Locale::Codes::Retired{'country'}{'dom'}{'name'} = {
   q(ascension island) => [ q(ac), q(Ascension Island) ],
   q(bolivia, plurinational state of) => [ q(bo), q(Bolivia, Plurinational State of) ],
   q(bonaire, saint eustatius and saba) => [ q(bq), q(Bonaire, Saint Eustatius and Saba) ],
   q(bonaire, sint eustatius and saba) => [ q(BQ), q(Bonaire, Sint Eustatius and Saba) ],
   q(cape verde) => [ q(cv), q(Cape Verde) ],
   q(congo, the democratic republic of the) => [ q(cd), q(Congo, The Democratic Republic of the) ],
   q(east timor) => [ q(tp), q(East Timor) ],
   q(european union) => [ q(eu), q(European Union) ],
   q(falkland islands (malvinas)) => [ q(fk), q(Falkland Islands (Malvinas)) ],
   q(france, metropolitan) => [ q(FX), q(France, Metropolitan) ],
   q(holy see (the) [vatican city state]) => [ q(va), q(Holy See (The) [Vatican City State]) ],
   q(holy see (vatican city state)) => [ q(va), q(Holy See (Vatican City State)) ],
   q(iran, islamic republic of) => [ q(ir), q(Iran, Islamic Republic of) ],
   q(iran, the islamic republic of) => [ q(ir), q(Iran, The Islamic Republic of) ],
   q(korea, democratic people's republic of) => [ q(kp), q(Korea, Democratic People's Republic of) ],
   q(korea, republic of) => [ q(kr), q(Korea, Republic of) ],
   q(libyan arab jamahiriya) => [ q(LY), q(Libyan Arab Jamahiriya) ],
   q(micronesia, federated states of) => [ q(fm), q(Micronesia, Federated States of) ],
   q(micronesia, the federated states of) => [ q(fm), q(Micronesia, The Federated States of) ],
   q(moldova, republic of) => [ q(md), q(Moldova, Republic of) ],
   q(netherlands antilles) => [ q(an), q(Netherlands Antilles) ],
   q(portuguese timor) => [ q(TP), q(Portuguese Timor) ],
   q(portuguese timor ) => [ q(TP), q(Portuguese Timor ) ],
   q(saint helena, ascension and tristan da cunha) => [ q(SH), q(Saint Helena, Ascension and Tristan da Cunha) ],
   q(saint martin) => [ q(MF), q(Saint Martin) ],
   q(soviet union) => [ q(SU), q(Soviet Union) ],
   q(soviet union ) => [ q(SU), q(Soviet Union ) ],
   q(taiwan) => [ q(TW), q(Taiwan) ],
   q(taiwan, province of china) => [ q(tw), q(Taiwan, Province of China) ],
   q(united kingdom) => [ q(gb), q(United Kingdom) ],
   q(united states) => [ q(us), q(United States) ],
   q(ussr) => [ q(su), q(USSR) ],
   q(venezuela, bolivarian republic of) => [ q(ve), q(Venezuela, Bolivarian Republic of) ],
   q(venezuela, bolivarian republic of ) => [ q(ve), q(Venezuela, Bolivarian Republic of ) ],
   q(virgin islands, british) => [ q(vg), q(Virgin Islands, British) ],
   q(virgin islands, u.s.) => [ q(vi), q(Virgin Islands, U.S.) ],
   q(yugoslavia ) => [ q(YU), q(Yugoslavia ) ],
};

$Locale::Codes::Retired{'country'}{'numeric'}{'name'} = {
   q(antarctica) => [ q(010), q(Antarctica) ],
   q(bolivia) => [ q(068), q(Bolivia) ],
   q(bolivia (plurinational state of)) => [ q(068), q(Bolivia (Plurinational State of)) ],
   q(bolivia, plurinational state of) => [ q(068), q(Bolivia, Plurinational State of) ],
   q(bonaire, saint eustatius and saba) => [ q(535), q(Bonaire, Saint Eustatius and Saba) ],
   q(bouvet island) => [ q(074), q(Bouvet Island) ],
   q(british indian ocean territory) => [ q(086), q(British Indian Ocean Territory) ],
   q(british virgin islands) => [ q(092), q(British Virgin Islands) ],
   q(cabo verde) => [ q(132), q(Cabo Verde) ],
   q(cape verde) => [ q(132), q(Cape Verde) ],
   q(channel islands) => [ q(830), q(Channel Islands) ],
   q(china, hong kong special administrative region) => [ q(344), q(China, Hong Kong Special Administrative Region) ],
   q(china, macao special administrative region) => [ q(446), q(China, Macao Special Administrative Region) ],
   q(christmas island) => [ q(162), q(Christmas Island) ],
   q(cocos (keeling) islands) => [ q(166), q(Cocos (Keeling) Islands) ],
   q(congo, the democratic republic of the) => [ q(180), q(Congo, The Democratic Republic of the) ],
   q(democratic people's republic of korea) => [ q(408), q(Democratic People's Republic of Korea) ],
   q(democratic republic of the congo) => [ q(180), q(Democratic Republic of the Congo) ],
   q(east timor) => [ q(626), q(East Timor) ],
   q(faeroe islands) => [ q(234), q(Faeroe Islands) ],
   q(falkland islands (malvinas)) => [ q(238), q(Falkland Islands (Malvinas)) ],
   q(faroe islands) => [ q(234), q(Faroe Islands) ],
   q(france, metropolitan) => [ q(249), q(France, Metropolitan) ],
   q(french southern and antarctic lands) => [ q(260), q(French Southern and Antarctic Lands) ],
   q(french southern territories) => [ q(260), q(French Southern Territories) ],
   q(heard island and mcdonald islands) => [ q(334), q(Heard Island and Mcdonald Islands) ],
   q(holy see) => [ q(336), q(Holy See) ],
   q(holy see (the) [vatican city state]) => [ q(336), q(Holy See (The) [Vatican City State]) ],
   q(holy see (vatican city state)) => [ q(336), q(Holy See (Vatican City State)) ],
   q(hong kong) => [ q(344), q(Hong Kong) ],
   q(hong kong special administrative region of china) => [ q(344), q(Hong Kong Special Administrative Region of China) ],
   q(iran (islamic republic of)) => [ q(364), q(Iran (Islamic Republic of)) ],
   q(iran, islamic republic of) => [ q(364), q(Iran, Islamic Republic of) ],
   q(iran, the islamic republic of) => [ q(364), q(Iran, The Islamic Republic of) ],
   q(kazakstan) => [ q(398), q(Kazakstan) ],
   q(korea, democratic people's republic of) => [ q(408), q(Korea, Democratic People's Republic of) ],
   q(korea, republic of) => [ q(410), q(Korea, Republic of) ],
   q(libyan arab jamahiriya) => [ q(434), q(Libyan Arab Jamahiriya) ],
   q(macao) => [ q(446), q(Macao) ],
   q(macao special administrative region of china) => [ q(446), q(Macao Special Administrative Region of China) ],
   q(macau) => [ q(446), q(Macau) ],
   q(macedonia, the former yugoslav republic of) => [ q(807), q(Macedonia, the Former Yugoslav Republic of) ],
   q(micronesia (federated states of)) => [ q(583), q(Micronesia (Federated States of)) ],
   q(micronesia, federated states of) => [ q(583), q(Micronesia, Federated States of) ],
   q(micronesia, the federated states of) => [ q(583), q(Micronesia, The Federated States of) ],
   q(moldova, republic of) => [ q(498), q(Moldova, Republic of) ],
   q(netherlands antilles) => [ q(530), q(Netherlands Antilles) ],
   q(palestinian territory, occupied) => [ q(275), q(Palestinian Territory, Occupied) ],
   q(republic of korea) => [ q(410), q(Republic of Korea) ],
   q(republic of moldova) => [ q(498), q(Republic of Moldova) ],
   q(saint helena) => [ q(654), q(Saint Helena) ],
   q(saint-barthelemy) => [ q(652), q(Saint-Barthelemy) ],
   q(saint-martin (french part)) => [ q(663), q(Saint-Martin (French part)) ],
   q(sark) => [ q(680), q(Sark) ],
   q(serbia and montenegro) => [ q(891), q(Serbia and Montenegro) ],
   q(south georgia and the islands) => [ q(239), q(South Georgia and the Islands) ],
   q(south georgia and the south sandwich islands) => [ q(239), q(South Georgia and the South Sandwich Islands) ],
   q(state of palestine) => [ q(275), q(State of Palestine) ],
   q(svalbard and jan mayen) => [ q(744), q(Svalbard and Jan Mayen) ],
   q(svalbard and jan mayen islands) => [ q(744), q(Svalbard and Jan Mayen Islands) ],
   q(taiwan) => [ q(158), q(Taiwan) ],
   q(taiwan, province of china) => [ q(158), q(Taiwan, Province of China) ],
   q(tanzania, united republic of) => [ q(834), q(Tanzania, United Republic of) ],
   q(the former yugoslav republic of macedonia) => [ q(807), q(The former Yugoslav Republic of Macedonia) ],
   q(united kingdom) => [ q(826), q(United Kingdom) ],
   q(united kingdom of great britain and northern ireland) => [ q(826), q(United Kingdom of Great Britain and Northern Ireland) ],
   q(united republic of tanzania) => [ q(834), q(United Republic of Tanzania) ],
   q(united states) => [ q(840), q(United States) ],
   q(united states minor outlying islands) => [ q(581), q(United States Minor Outlying Islands) ],
   q(united states of america) => [ q(840), q(United States of America) ],
   q(united states virgin islands) => [ q(850), q(United States Virgin Islands) ],
   q(venezuela) => [ q(862), q(Venezuela) ],
   q(venezuela (bolivarian republic of)) => [ q(862), q(Venezuela (Bolivarian Republic of)) ],
   q(venezuela, bolivarian republic of ) => [ q(862), q(Venezuela, Bolivarian Republic of ) ],
   q(vietnam) => [ q(704), q(Vietnam) ],
   q(virgin islands, british) => [ q(092), q(Virgin Islands, British) ],
   q(virgin islands, u.s.) => [ q(850), q(Virgin Islands, U.S.) ],
   q(wallis and futuna) => [ q(876), q(Wallis and Futuna) ],
   q(wallis and futuna islands) => [ q(876), q(Wallis and Futuna Islands) ],
   q(yugoslavia) => [ q(891), q(Yugoslavia) ],
   q(zaire) => [ q(180), q(Zaire) ],
};


1;
